'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.default = QuickSearchForm;

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _reactIntl = require('react-intl');

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _pickBy = require('lodash/pickBy');

var _pickBy2 = _interopRequireDefault(_pickBy);

var _cspaceInput = require('cspace-input');

var _permissionHelpers = require('../../helpers/permissionHelpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var QuickSearchInput = _cspaceInput.components.QuickSearchInput;


var messages = (0, _reactIntl.defineMessages)({
  placeholder: {
    'id': 'quickSearchForm.placeholder',
    'defaultMessage': 'Search'
  },
  search: {
    'id': 'quickSearchForm.search',
    'defaultMessage': 'Search'
  }
});

var getRecordTypes = function getRecordTypes(config, perms) {
  var recordTypes = config.recordTypes;


  return (0, _pickBy2.default)(recordTypes, function (recordTypeConfig, name) {
    var serviceType = (0, _get2.default)(recordTypeConfig, ['serviceConfig', 'serviceType']);

    if (serviceType === 'object' || serviceType === 'procedure' || serviceType === 'authority') {
      return (0, _permissionHelpers.canList)(name, perms);
    }

    return true;
  });
};

var propTypes = {
  intl: _reactIntl.intlShape,
  config: _propTypes2.default.object,
  perms: _propTypes2.default.instanceOf(_immutable2.default.Map)
};

function QuickSearchForm(props) {
  var intl = props.intl,
      config = props.config,
      perms = props.perms,
      remainingProps = _objectWithoutProperties(props, ['intl', 'config', 'perms']);

  var formatRecordTypeLabel = function formatRecordTypeLabel(name, recordTypeConfig) {
    return intl.formatMessage(recordTypeConfig.messages.record.collectionName);
  };

  var formatVocabularyLabel = function formatVocabularyLabel(name, vocabularyConfig) {
    return intl.formatMessage(vocabularyConfig.messages.name);
  };

  return _react2.default.createElement(
    'fieldset',
    null,
    _react2.default.createElement(QuickSearchInput, _extends({}, remainingProps, {
      formatRecordTypeLabel: formatRecordTypeLabel,
      formatVocabularyLabel: formatVocabularyLabel,
      placeholder: intl.formatMessage(messages.placeholder),
      recordTypes: getRecordTypes(config, perms),
      searchButtonLabel: intl.formatMessage(messages.search)
    }))
  );
}

QuickSearchForm.propTypes = propTypes;