'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reactIntl = require('react-intl');

var _cspaceRefname = require('cspace-refname');

var _configHelpers = require('../../helpers/configHelpers');

exports.default = function () {
  return {
    listTypes: {
      authRef: {
        listNodeName: 'ns3:authority-ref-list',
        itemNodeName: 'authority-ref-item',
        // NB: This is a list of terms, not records: a record may appear multiple times in the list,
        // if multiple synonyms are used by the record. The messages reflect this.
        messages: (0, _reactIntl.defineMessages)({
          resultCount: {
            'id': 'list.authRef.resultCount',
            'defaultMessage': '{totalItems, plural, =0 {No terms} one {1 term} other {{startNum}\u2013{endNum} of {totalItems} terms}} found'
          },
          searching: {
            'id': 'list.authRef.searching',
            'defaultMessage': 'Finding terms...'
          }
        }),
        getItemLocationPath: function getItemLocationPath(item, _ref) {
          var config = _ref.config;

          var refName = item.get('refName');
          var servicePath = (0, _cspaceRefname.getServicePath)(refName);
          var recordTypeConfig = (0, _configHelpers.getRecordTypeConfigByServicePath)(config, servicePath);

          if (recordTypeConfig) {
            var vocabularyShortID = (0, _cspaceRefname.getVocabularyShortID)(refName);

            var vocabularyConfig = (0, _configHelpers.getVocabularyConfigByShortID)(recordTypeConfig, vocabularyShortID);

            if (vocabularyConfig) {
              var csid = item.get('csid');

              if (csid) {
                return '/record/' + recordTypeConfig.name + '/' + vocabularyConfig.name + '/' + csid;
              }
            }
          }

          return null;
        }
      }
    }
  };
};