'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _reactIntl = require('react-intl');

var _cspaceRefname = require('cspace-refname');

var _configHelpers = require('../../helpers/configHelpers');

exports.default = function () {
  return {
    listTypes: {
      common: {
        listNodeName: 'ns2:abstract-common-list',
        itemNodeName: 'list-item',
        messages: (0, _reactIntl.defineMessages)({
          resultCount: {
            'id': 'list.common.resultCount',
            'defaultMessage': '{totalItems, plural, =0 {No records} one {1 record} other {{startNum}\u2013{endNum} of {totalItems} records}} found'
          },
          searching: {
            'id': 'list.common.searching',
            'defaultMessage': 'Finding records...'
          }
        }),
        getItemLocationPath: function getItemLocationPath(item, _ref) {
          var config = _ref.config,
              searchDescriptor = _ref.searchDescriptor;

          var docType = item.get('docType');
          var refName = item.get('refName');
          var csid = item.get('csid');

          var recordTypeConfig = void 0;

          if (docType) {
            // This is a search on multiple record types, which will contain the record type of each
            // item in the result.

            recordTypeConfig = (0, _configHelpers.getRecordTypeConfigByServiceObjectName)(config, docType);
          } else {
            // This is a search on a single record type, so the record type of all items is in the
            // search descriptor.

            recordTypeConfig = config.recordTypes[searchDescriptor.get('recordType')];
          }

          if (recordTypeConfig) {
            if (recordTypeConfig.serviceConfig.serviceType === 'authority') {
              var vocabularyShortID = (0, _cspaceRefname.getVocabularyShortID)(refName);

              var vocabularyConfig = (0, _configHelpers.getVocabularyConfigByShortID)(recordTypeConfig, vocabularyShortID);

              if (vocabularyConfig) {
                return '/record/' + recordTypeConfig.name + '/' + vocabularyConfig.name + '/' + csid;
              }
            } else {
              return '/record/' + recordTypeConfig.name + '/' + csid;
            }
          }

          return null;
        }
      }
    }
  };
};